//
//  CameraService.swift
//  macamera
//
//  Created by Maxime on 06/05/2025.
//

import AVFoundation
import Foundation

class CameraService {
    
    var session: AVCaptureSession?
    var delegate: AVCapturePhotoCaptureDelegate?
    
    let output = AVCapturePhotoOutput()
    let previewLayer = AVCaptureVideoPreviewLayer()
}

// MARK: - Public Interface

extension CameraService {
    
    func start(delegate: AVCapturePhotoCaptureDelegate) async {
        self.delegate = delegate
        
        let accessIsGranted = await self.checkPermission()
        self.setupCaptureSession(accessIsGranted: accessIsGranted)
    }
    
    func capturePhoto(with settings: AVCapturePhotoSettings = AVCapturePhotoSettings()) {
        guard let delegate = delegate else { return }
        output.capturePhoto(with: settings, delegate: delegate)
    }
}

// MARK: - Initialization

extension CameraService {
    
    private func checkPermission() async -> Bool {
        switch AVCaptureDevice.authorizationStatus(for: .video) {
        case .authorized:
            return true
        case .notDetermined:
            return await AVCaptureDevice.requestAccess(for: .video)
        default:
            return false
        }
    }
    
    private func setupCaptureSession(accessIsGranted: Bool) {
        guard accessIsGranted,
              let dualCamera = AVCaptureDevice.default(.builtInDualWideCamera, for: .video, position: .back),
              let input = try? AVCaptureDeviceInput(device: dualCamera) else {
                  return
              }
        
        let session = AVCaptureSession()
        guard session.canAddInput(input), session.canAddOutput(output) else {
            return
        }
        
        session.addInput(input)
        session.addOutput(output)
        output.connection(with: .video)?.videoRotationAngle = 90
        
        previewLayer.videoGravity = .resizeAspectFill
        previewLayer.session = session
        
        session.startRunning()
        self.session = session
    }
}
