//
//  CameraView.swift
//  macamera
//
//  Created by Maxime on 06/05/2025.
//

import SwiftUI
import UIKit
import AVFoundation

struct CameraView: UIViewControllerRepresentable {
    typealias UIViewControllerType = UIViewController
    typealias ProcessingPhotoResult = Result<AVCapturePhoto, Error>
    
    let cameraService: CameraService
    let didFinishProcessingPhoto: (ProcessingPhotoResult) -> Void
    
    func makeUIViewController(context: Context) -> UIViewController {
        Task.detached {
            await cameraService.start(delegate: context.coordinator)
        }
        
        let viewController = UIViewController()
        viewController.view.backgroundColor = .black
        viewController.view.layer.addSublayer(cameraService.previewLayer)
        cameraService.previewLayer.frame = viewController.view.bounds
        return viewController
    }
    
    func makeCoordinator() -> Coordinator {
        Coordinator(self, didFinishProcessingPhoto: didFinishProcessingPhoto)
    }
    
    func updateUIViewController(_ uiViewController: UIViewController, context: Context) { }
    
    class Coordinator: NSObject, AVCapturePhotoCaptureDelegate {
        let parent: CameraView
        private var didFinishProcessingPhoto: (ProcessingPhotoResult) -> Void
        
        init(_ parent: CameraView, didFinishProcessingPhoto: @escaping (ProcessingPhotoResult) -> Void) {
            self.parent = parent
            self.didFinishProcessingPhoto = didFinishProcessingPhoto
        }
        
        func photoOutput(_ output: AVCapturePhotoOutput, didFinishProcessingPhoto photo: AVCapturePhoto, error: (any Error)?) {
            if let error = error {
                didFinishProcessingPhoto(.failure(error))
                return
            }
            didFinishProcessingPhoto(.success(photo))
        }
    }
}
